@@ -13,10 +13,9 @@
 #    See the License for the specific language governing permissions and
 #    limitations under the License.
 
-import json
-
-import unittest2
+from oslo_serialization import jsonutils
 
+from mistralclient.api import base as api_base
 from mistralclient.api.v2 import executions
 from mistralclient.tests.unit.v2 import base
 
@@ -26,6 +25,7 @@ EXEC = {
     'id': "123",
     'workflow_id': '123e4567-e89b-12d3-a456-426655440000',
     'workflow_name': 'my_wf',
+    'workflow_namespace': '',
     'description': '',
     'state': 'RUNNING',
     'input': {
@@ -40,6 +40,7 @@ SUB_WF_EXEC = {
     'id': "456",
     'workflow_id': '123e4567-e89b-12d3-a456-426655440000',
     'workflow_name': 'my_sub_wf',
+    'workflow_namespace': '',
     'task_execution_id': "abc",
     'description': '',
     'state': 'RUNNING',
@@ -51,21 +52,27 @@ SUB_WF_EXEC = {
     }
 }
 
+SOURCE_EXEC = EXEC
+SOURCE_EXEC['source_execution_id'] = EXEC['workflow_id']
 URL_TEMPLATE = '/executions'
 URL_TEMPLATE_ID = '/executions/%s'
 
 
 class TestExecutionsV2(base.BaseClientV2Test):
     def test_create(self):
-        mock = self.mock_http_post(content=EXEC)
+        self.requests_mock.post(self.TEST_URL + URL_TEMPLATE,
+                                json=EXEC,
+                                status_code=201)
+
         body = {
             'workflow_name': EXEC['workflow_name'],
             'description': '',
-            'input': json.dumps(EXEC['input']),
+            'input': jsonutils.dumps(EXEC['input'])
         }
 
         ex = self.executions.create(
             EXEC['workflow_name'],
+            EXEC['workflow_namespace'],
             EXEC['input']
         )
 
@@ -76,20 +83,22 @@ class TestExecutionsV2(base.BaseClientV2Test):
             ex.to_dict()
         )
 
-        self.assertEqual(URL_TEMPLATE, mock.call_args[0][0])
-        self.assertDictEqual(body, json.loads(mock.call_args[0][1]))
+        self.assertDictEqual(body, self.requests_mock.last_request.json())
 
     def test_create_with_workflow_id(self):
-        mock = self.mock_http_post(content=EXEC)
+        self.requests_mock.post(self.TEST_URL + URL_TEMPLATE,
+                                json=EXEC,
+                                status_code=201)
+
         body = {
             'workflow_id': EXEC['workflow_id'],
             'description': '',
-            'input': json.dumps(EXEC['input']),
+            'input': jsonutils.dumps(EXEC['input'])
         }
 
         ex = self.executions.create(
             EXEC['workflow_id'],
-            EXEC['input']
+            workflow_input=EXEC['input']
         )
 
         self.assertIsNotNone(ex)
@@ -99,24 +108,40 @@ class TestExecutionsV2(base.BaseClientV2Test):
             ex.to_dict()
         )
 
-        self.assertEqual(URL_TEMPLATE, mock.call_args[0][0])
-        self.assertDictEqual(body, json.loads(mock.call_args[0][1]))
+        self.assertDictEqual(body, self.requests_mock.last_request.json())
 
-    @unittest2.expectedFailure
-    def test_create_failure1(self):
-        self.mock_http_post(content=EXEC)
-        self.executions.create("")
+    def test_create_with_source_execution_id(self):
+        self.requests_mock.post(self.TEST_URL + URL_TEMPLATE,
+                                json=SOURCE_EXEC,
+                                status_code=201)
 
-    @unittest2.expectedFailure
-    def test_create_failure2(self):
-        self.mock_http_post(content=EXEC)
-        self.executions.create(
-            EXEC['workflow_name'],
-            list('343', 'sdfsd')
+        body = {
+            'description': '',
+            'source_execution_id': SOURCE_EXEC['source_execution_id']
+        }
+
+        ex = self.executions.create(
+            source_execution_id=SOURCE_EXEC['source_execution_id']
         )
 
+        self.assertIsNotNone(ex)
+
+        self.assertDictEqual(
+            executions.Execution(self.executions, SOURCE_EXEC).to_dict(),
+            ex.to_dict()
+        )
+
+        self.assertDictEqual(body, self.requests_mock.last_request.json())
+
+    def test_create_failure1(self):
+        self.requests_mock.post(self.TEST_URL + URL_TEMPLATE,
+                                json=EXEC,
+                                status_code=201)
+        self.assertRaises(api_base.APIException, self.executions.create, '')
+
     def test_update(self):
-        mock = self.mock_http_put(content=EXEC)
+        url = self.TEST_URL + URL_TEMPLATE_ID % EXEC['id']
+        self.requests_mock.put(url, json=EXEC)
         body = {
             'state': EXEC['state']
         }
@@ -130,11 +155,11 @@ class TestExecutionsV2(base.BaseClientV2Test):
             ex.to_dict()
         )
 
-        self.assertEqual(URL_TEMPLATE_ID % EXEC['id'], mock.call_args[0][0])
-        self.assertDictEqual(body, json.loads(mock.call_args[0][1]))
+        self.assertDictEqual(body, self.requests_mock.last_request.json())
 
     def test_update_env(self):
-        mock = self.mock_http_put(content=EXEC)
+        url = self.TEST_URL + URL_TEMPLATE_ID % EXEC['id']
+        self.requests_mock.put(url, json=EXEC)
         body = {
             'state': EXEC['state'],
             'params': {
@@ -155,11 +180,11 @@ class TestExecutionsV2(base.BaseClientV2Test):
             ex.to_dict()
         )
 
-        self.assertEqual(URL_TEMPLATE_ID % EXEC['id'], mock.call_args[0][0])
-        self.assertDictEqual(body, json.loads(mock.call_args[0][1]))
+        self.assertDictEqual(body, self.requests_mock.last_request.json())
 
     def test_list(self):
-        mock = self.mock_http_get(content={'executions': [EXEC, SUB_WF_EXEC]})
+        self.requests_mock.get(self.TEST_URL + URL_TEMPLATE,
+                               json={'executions': [EXEC, SUB_WF_EXEC]})
 
         execution_list = self.executions.list()
 
@@ -175,12 +200,10 @@ class TestExecutionsV2(base.BaseClientV2Test):
             execution_list[1].to_dict()
         )
 
-        mock.assert_called_once_with(URL_TEMPLATE)
-
     def test_list_with_pagination(self):
-        mock = self.mock_http_get(
-            content={'executions': [EXEC], 'next': '/executions?fake'}
-        )
+        self.requests_mock.get(self.TEST_URL + URL_TEMPLATE,
+                               json={'executions': [EXEC],
+                                     'next': '/executions?fake'})
 
         execution_list = self.executions.list(
             limit=1,
@@ -190,13 +213,28 @@ class TestExecutionsV2(base.BaseClientV2Test):
 
         self.assertEqual(1, len(execution_list))
 
+        last_request = self.requests_mock.last_request
+
         # The url param order is unpredictable.
-        self.assertIn('limit=1', mock.call_args[0][0])
-        self.assertIn('sort_keys=created_at', mock.call_args[0][0])
-        self.assertIn('sort_dirs=asc', mock.call_args[0][0])
+        self.assertEqual(['1'], last_request.qs['limit'])
+        self.assertEqual(['created_at'], last_request.qs['sort_keys'])
+        self.assertEqual(['asc'], last_request.qs['sort_dirs'])
+
+    def test_list_with_no_limit(self):
+        self.requests_mock.get(self.TEST_URL + URL_TEMPLATE,
+                               json={'executions': [EXEC]})
+
+        execution_list = self.executions.list(limit=-1)
+
+        self.assertEqual(1, len(execution_list))
+
+        last_request = self.requests_mock.last_request
+
+        self.assertNotIn('limit', last_request.qs)
 
     def test_get(self):
-        mock = self.mock_http_get(content=EXEC)
+        url = self.TEST_URL + URL_TEMPLATE_ID % EXEC['id']
+        self.requests_mock.get(url, json=EXEC)
 
         ex = self.executions.get(EXEC['id'])
 
@@ -205,10 +243,9 @@ class TestExecutionsV2(base.BaseClientV2Test):
             ex.to_dict()
         )
 
-        mock.assert_called_once_with(URL_TEMPLATE_ID % EXEC['id'])
-
     def test_get_sub_wf_ex(self):
-        mock = self.mock_http_get(content=SUB_WF_EXEC)
+        url = self.TEST_URL + URL_TEMPLATE_ID % SUB_WF_EXEC['id']
+        self.requests_mock.get(url, json=SUB_WF_EXEC)
 
         ex = self.executions.get(SUB_WF_EXEC['id'])
 
@@ -217,11 +254,28 @@ class TestExecutionsV2(base.BaseClientV2Test):
             ex.to_dict()
         )
 
-        mock.assert_called_once_with(URL_TEMPLATE_ID % SUB_WF_EXEC['id'])
+    def test_delete_with_force(self):
+        url = self.TEST_URL + URL_TEMPLATE_ID % EXEC['id']
+        self.requests_mock.delete(url, status_code=204)
+
+        self.executions.delete(EXEC['id'], force=True)
 
     def test_delete(self):
-        mock = self.mock_http_delete(status_code=204)
+        url = self.TEST_URL + URL_TEMPLATE_ID % EXEC['id']
+        self.requests_mock.delete(url, status_code=204)
 
         self.executions.delete(EXEC['id'])
 
-        mock.assert_called_once_with(URL_TEMPLATE_ID % EXEC['id'])
+    def test_report(self):
+        url = self.TEST_URL + URL_TEMPLATE_ID % EXEC['id'] + '/report'
+
+        expected_json = {
+            'root_workflow_execution': {},
+            'statistics': {}
+        }
+
+        self.requests_mock.get(url, json=expected_json)
+
+        report = self.executions.get_report(EXEC['id'])
+
+        self.assertDictEqual(expected_json, report)
